@@ -33,7 +33,9 @@ remote_cmd, remote_schema = None, None
 force = None
 select_opts = []
 select_files = []
-user_mapping_filename, group_mapping_filename = None, None
+user_mapping_filename, group_mapping_filename, preserve_numerical_ids = \
+					   None, None, None
+
 # These are global because they are set while we are trying to figure
 # whether to restore or to backup
 restore_root, restore_index, restore_root_set = None, None, 0
@@ -43,7 +45,9 @@ def parse_cmdlineoptions(arglist):
 	"""Parse argument list and set global preferences"""
 	global args, action, create_full_path, force, restore_timestr, remote_cmd
 	global remote_schema, remove_older_than_string
-	global user_mapping_filename, group_mapping_filename
+	global user_mapping_filename, group_mapping_filename, \
+		   preserve_numerical_ids
+
 	def sel_fl(filename):
 		"""Helper function for including/excluding filelists below"""
 		try: return open(filename, "r")
@@ -54,8 +58,7 @@ def parse_cmdlineoptions(arglist):
 		return rpath.RPath(Globals.local_connection, path).normalize().path
 
 	try: optlist, args = getopt.getopt(arglist, "blr:sv:V",
-		 ["backup-mode", "calculate-average", "carbonfile",
-		  "check-destination-dir",
+		 ["backup-mode", "calculate-average", "check-destination-dir",
 		  "compare", "compare-at-time=", "create-full-path",
 		  "current-time=", "exclude=", "exclude-device-files",
 		  "exclude-fifos", "exclude-filelist=",
@@ -74,7 +77,8 @@ def parse_cmdlineoptions(arglist):
 		  "no-compression-regexp=", "no-eas", "no-file-statistics",
 		  "no-hard-links", "null-separator",
 		  "override-chars-to-quote=", "parsable-output",
-		  "print-statistics", "remote-cmd=", "remote-schema=",
+		  "preserve-numerical-ids", "print-statistics",
+		  "remote-cmd=", "remote-schema=",
 		  "remove-older-than=", "restore-as-of=", "restrict=",
 		  "restrict-read-only=", "restrict-update-only=", "server",
 		  "ssh-no-compression", "terminal-verbosity=", "test-server",
@@ -157,6 +161,7 @@ def parse_cmdlineoptions(arglist):
 		elif opt == "--override-chars-to-quote":
 			Globals.set('chars_to_quote', arg)
 		elif opt == "--parsable-output": Globals.set('parsable_output', 1)
+		elif opt == "--preserve-numerical-ids": preserve_numerical_ids = 1
 		elif opt == "--print-statistics": Globals.set('print_statistics', 1)
 		elif opt == "-r" or opt == "--restore-as-of":
 			restore_timestr, action = arg, "restore-as-of"
@@ -234,7 +239,8 @@ def misc_setup(rps):
 
 def init_user_group_mapping(destination_conn):
 	"""Initialize user and group mapping on destination connection"""
-	global user_mapping_filename, group_mapping_filename
+	global user_mapping_filename, group_mapping_filename, \
+		   preserve_numerical_ids
 	def get_string_from_file(filename):
 		if not filename: return None
 		rp = rpath.RPath(Globals.local_connection, filename)
@@ -243,9 +249,11 @@ def init_user_group_mapping(destination_conn):
 			Log.FatalError("Error '%s' reading mapping file '%s'" %
 						   (str(e), filename))
 	user_mapping_string = get_string_from_file(user_mapping_filename)
-	destination_conn.user_group.init_user_mapping(user_mapping_string)
+	destination_conn.user_group.init_user_mapping(user_mapping_string,
+												  preserve_numerical_ids)
 	group_mapping_string = get_string_from_file(group_mapping_filename)
-	destination_conn.user_group.init_group_mapping(group_mapping_string)
+	destination_conn.user_group.init_group_mapping(group_mapping_string,
+												   preserve_numerical_ids)
 
 def take_action(rps):
 	"""Do whatever action says"""
@@ -293,7 +301,7 @@ def Backup(rpin, rpout):
 	SetConnections.BackupInitConnections(rpin.conn, rpout.conn)
 	backup_check_dirs(rpin, rpout)
 	backup_set_rbdir(rpin, rpout)
-	backup_set_fs_globals(rpin, rpout)
+	rpout.conn.fs_abilities.backup_set_globals(rpin)
 	if Globals.chars_to_quote: rpout = backup_quoted_rpaths(rpout)
 	init_user_group_mapping(rpout.conn)
 	backup_final_init(rpout)
@@ -345,7 +353,6 @@ def backup_check_dirs(rpin, rpout):
 def backup_set_rbdir(rpin, rpout):
 	"""Initialize data dir and logging"""
 	global incdir
-	SetConnections.UpdateGlobal('rbdir', Globals.rbdir)
 	incdir = Globals.rbdir.append_path("increments")
 
 	assert rpout.lstat(), (rpout.path, rpout.lstat())
@@ -362,6 +369,7 @@ want to update or overwrite it, run rdiff-backup with the --force
 option.""" % rpout.path)
 
 	if not Globals.rbdir.lstat(): Globals.rbdir.mkdir()
+	SetConnections.UpdateGlobal('rbdir', Globals.rbdir)
 
 def backup_warn_if_infinite_regress(rpin, rpout):
 	"""Warn user if destination area contained in source area"""
@@ -399,53 +407,6 @@ def backup_final_init(rpout):
 	inc_base = Globals.rbdir.append_path("increments")
 	if not inc_base.lstat(): inc_base.mkdir()
 
-def backup_set_fs_globals(rpin, rpout):
-	"""Use fs_abilities to set the globals that depend on filesystem"""
-	def update_triple(src_support, dest_support, attr_triple):
-		"""Update global settings for feature based on fsa results"""
-		active_attr, write_attr, conn_attr = attr_triple
-		if Globals.get(active_attr) == 0: return # don't override 0
-		for attr in attr_triple: SetConnections.UpdateGlobal(attr, None)
-		if not src_support: return # if source doesn't support, nothing
-		SetConnections.UpdateGlobal(active_attr, 1)
-		rpin.conn.Globals.set_local(conn_attr, 1)
-		if dest_support:
-			SetConnections.UpdateGlobal(write_attr, 1)
-			rpout.conn.Globals.set_local(conn_attr, 1)
-
-	src_fsa = rpin.conn.fs_abilities.get_fsabilities_readonly('source', rpin)
-	Log(str(src_fsa), 4)
-	dest_fsa = rpout.conn.fs_abilities.get_fsabilities_readwrite(
-		'destination', Globals.rbdir, 1, Globals.chars_to_quote)
-	Log(str(dest_fsa), 4)
-
-	update_triple(src_fsa.eas, dest_fsa.eas,
-				  ('eas_active', 'eas_write', 'eas_conn'))
-	update_triple(src_fsa.acls, dest_fsa.acls,
-				  ('acls_active', 'acls_write', 'acls_conn'))
-	update_triple(src_fsa.resource_forks, dest_fsa.resource_forks,
-				  ('resource_forks_active', 'resource_forks_write',
-				   'resource_forks_conn'))
-
-	update_triple(src_fsa.carbonfile, dest_fsa.carbonfile,
-				  ('carbonfile_active', 'carbonfile_write', 'carbonfile_conn'))
-	if src_fsa.carbonfile and not Globals.carbonfile_active:
-		Log("Source may have carbonfile support, but support defaults to "
-			"off.\n  Use --carbonfile to enable.", 5)
-
-	if Globals.never_drop_acls and not Globals.acls_active:
-		Log.FatalError("--never-drop-acls specified, but ACL support\n"
-					   "disabled on destination filesystem")
-
-	if Globals.preserve_hardlinks != 0:
-		SetConnections.UpdateGlobal('preserve_hardlinks', dest_fsa.hardlinks)
-	SetConnections.UpdateGlobal('fsync_directories', dest_fsa.fsync_dirs)
-	SetConnections.UpdateGlobal('change_ownership', dest_fsa.ownership)
-	SetConnections.UpdateGlobal('chars_to_quote', dest_fsa.chars_to_quote)
-	if not dest_fsa.high_perms:
-		SetConnections.UpdateGlobal('permission_mask', 0777)
-	if Globals.chars_to_quote: FilenameMapping.set_init_quote_vals()
-	
 def backup_touch_curmirror_local(rpin, rpout):
 	"""Make a file like current_mirror.time.data to record time
 
@@ -486,11 +447,9 @@ def Restore(src_rp, dest_rp, restore_as_of = None):
 	mirror file), dest_rp should be the target rp to be written.
 
 	"""
-	if not restore_root_set and not restore_set_root(src_rp):
-		Log.FatalError("Could not find rdiff-backup repository at "
-					   + src_rp.path)
+	if not restore_root_set: assert restore_set_root(src_rp)
 	restore_check_paths(src_rp, dest_rp, restore_as_of)
-	restore_set_fs_globals(dest_rp)
+	dest_rp.conn.fs_abilities.restore_set_globals(dest_rp)
 	init_user_group_mapping(dest_rp.conn)
 	src_rp = restore_init_quoting(src_rp)
 	restore_check_backup_dir(restore_root, src_rp, restore_as_of)
@@ -515,51 +474,6 @@ def restore_init_quoting(src_rp):
 		'rbdir', FilenameMapping.get_quotedrpath(Globals.rbdir))
 	return FilenameMapping.get_quotedrpath(src_rp)
 
-def restore_set_fs_globals(target):
-	"""Use fs_abilities to set the globals that depend on filesystem"""
-	def update_triple(src_support, dest_support, attr_triple):
-		"""Update global settings for feature based on fsa results"""
-		active_attr, write_attr, conn_attr = attr_triple
-		if Globals.get(active_attr) == 0: return # don't override 0
-		for attr in attr_triple: SetConnections.UpdateGlobal(attr, None)
-		if not dest_support: return # if dest doesn't support, do nothing
-		SetConnections.UpdateGlobal(active_attr, 1)
-		target.conn.Globals.set_local(conn_attr, 1)
-		target.conn.Globals.set_local(write_attr, 1)
-		if src_support: Globals.rbdir.conn.Globals.set_local(conn_attr, 1)
-
-	target_fsa = target.conn.fs_abilities.get_fsabilities_readwrite(
-		'destination', target, 0, Globals.chars_to_quote)
-	Log(str(target_fsa), 4)
-	mirror_fsa = Globals.rbdir.conn.fs_abilities.get_fsabilities_restoresource(
-		Globals.rbdir)
-	Log(str(mirror_fsa), 4)
-
-	update_triple(mirror_fsa.eas, target_fsa.eas,
-				  ('eas_active', 'eas_write', 'eas_conn'))
-	update_triple(mirror_fsa.acls, target_fsa.acls,
-				  ('acls_active', 'acls_write', 'acls_conn'))
-	update_triple(mirror_fsa.resource_forks, target_fsa.resource_forks,
-				  ('resource_forks_active', 'resource_forks_write',
-				   'resource_forks_conn'))
-	update_triple(mirror_fsa.carbonfile, target_fsa.carbonfile,
-				  ('carbonfile_active', 'carbonfile_write', 'carbonfile_conn'))
-	if Globals.never_drop_acls and not Globals.acls_active:
-		Log.FatalError("--never-drop-acls specified, but ACL support\n"
-					   "disabled on destination filesystem")
-
-	if Globals.preserve_hardlinks != 0:
-		SetConnections.UpdateGlobal('preserve_hardlinks', target_fsa.hardlinks)
-	SetConnections.UpdateGlobal('change_ownership', target_fsa.ownership)
-	if not target_fsa.high_perms:
-		SetConnections.UpdateGlobal('permission_mask', 0777)
-
-	if Globals.chars_to_quote is None: # otherwise already overridden
-		if mirror_fsa.chars_to_quote:
-			SetConnections.UpdateGlobal('chars_to_quote',
-										mirror_fsa.chars_to_quote)
-		else: SetConnections.UpdateGlobal('chars_to_quote', "")
-
 def restore_set_select(mirror_rp, target):
 	"""Set the selection iterator on both side from command line args
 
@@ -595,14 +509,13 @@ def restore_start_log(rpin, target, time):
 	if Log.verbosity >= 3: Log.log_to_file(log_message)
 
 def restore_check_paths(rpin, rpout, restoreasof = None):
-	"""Make sure source and destination exist, and have appropriate type"""
+	"""Check paths and return pair of corresponding rps"""
 	if not restoreasof:
 		if not rpin.lstat():
 			Log.FatalError("Source file %s does not exist" % rpin.path)
 	if not force and rpout.lstat() and (not rpout.isdir() or rpout.listdir()):
 		Log.FatalError("Restore target %s already exists, "
 					   "specify --force to overwrite." % rpout.path)
-	if force and rpout.lstat() and not rpout.isdir(): rpout.delete()
 
 def restore_check_backup_dir(mirror_root, src_rp = None, restore_as_of = 1):
 	"""Make sure backup dir root rpin is in consistent state"""
@@ -657,6 +570,8 @@ def restore_set_root(rpin):
 
 	restore_root = parent_dir
 	Log("Using mirror root directory %s" % restore_root.path, 6)
+	if restore_root.conn is Globals.local_connection:
+		Security.reset_restrict_path(restore_root)
 	SetConnections.UpdateGlobal('rbdir',
 								restore_root.append_path("rdiff-backup-data"))
 	if not Globals.rbdir.isdir():
@@ -697,49 +612,10 @@ def require_root_set(rp):
 	if not restore_set_root(rp):
 		Log.FatalError(("Bad directory %s.\n" % (rp.path,)) +
 		  "It doesn't appear to be an rdiff-backup destination dir")
-	single_set_fs_globals(Globals.rbdir)
+	Globals.rbdir.conn.fs_abilities.single_set_globals(Globals.rbdir)
 	if Globals.chars_to_quote: return restore_init_quoting(rp)
 	else: return rp
 	
-def single_set_fs_globals(rbdir):
-	"""Use fs_abilities to set globals that depend on filesystem.
-
-	This is appropriate for listing increments, or any other operation
-	that depends only on the one file system.
-
-	"""
-	def update_triple(fsa_support, attr_triple):
-		"""Update global settings based on fsa result"""
-		active_attr, write_attr, conn_attr = attr_triple
-		if Globals.get(active_attr) == 0: return # don't override 0
-		for attr in attr_triple: SetConnections.UpdateGlobal(attr, None)
-		if not fsa_support: return
-		SetConnections.UpdateGlobal(active_attr, 1)
-		SetConnections.UpdateGlobal(write_attr, 1)
-		rbdir.conn.Globals.set_local(conn_attr, 1)
-
-	fsa = rbdir.conn.fs_abilities.get_fsabilities_readwrite('archive',
-								   rbdir, 1, Globals.chars_to_quote)
-	Log(str(fsa), 4)
-
-	update_triple(fsa.eas, ('eas_active', 'eas_write', 'eas_conn'))
-	update_triple(fsa.acls, ('acls_active', 'acls_write', 'acls_conn'))
-	update_triple(fsa.resource_forks,
-				  ('resource_forks_active', 'resource_forks_write',
-				   'resource_forks_conn'))
-	update_triple(fsa.carbonfile,
-				  ('carbonfile_active', 'carbonfile_write', 'carbonfile_conn'))
-
-	if Globals.preserve_hardlinks != 0:
-		SetConnections.UpdateGlobal('preserve_hardlinks', fsa.hardlinks)
-	SetConnections.UpdateGlobal('fsync_directories', fsa.fsync_dirs)
-	SetConnections.UpdateGlobal('change_ownership', fsa.ownership)
-	if not fsa.high_perms: SetConnections.UpdateGlobal('permission_mask', 0777)
-	SetConnections.UpdateGlobal('chars_to_quote', fsa.chars_to_quote)
-	if Globals.chars_to_quote:
-		for conn in Globals.connections:
-			conn.FilenameMapping.set_init_quote_vals()
-
 
 def ListIncrementSizes(rp):
 	"""Print out a summary of the increments """
@@ -760,24 +636,17 @@ def RemoveOlderThan(rootrp):
 	"""Remove all increment files older than a certain time"""
 	rootrp = require_root_set(rootrp)
 	rot_require_rbdir_base(rootrp)
-
-	time = rot_check_time(remove_older_than_string)
-	if time is None: return
-	Log("Actual remove older than time: %s" % (time,), 6)
-	manage.delete_earlier_than(Globals.rbdir, time)
-
-def rot_check_time(time_string):
-	"""Check remove older than time_string, return time in seconds"""
-	try: time = Time.genstrtotime(time_string)
+	try: time = Time.genstrtotime(remove_older_than_string)
 	except Time.TimeException, exc: Log.FatalError(str(exc))
+	timep = Time.timetopretty(time)
+	Log("Deleting increment(s) before %s" % timep, 4)
 
 	times_in_secs = [inc.getinctime() for inc in 
 		  restore.get_inclist(Globals.rbdir.append_path("increments"))]
 	times_in_secs = filter(lambda t: t < time, times_in_secs)
 	if not times_in_secs:
-		Log("No increments older than %s found, exiting." %
-			(Time.timetopretty(time),), 3)
-		return None
+		Log.FatalError("No increments older than %s found, exiting."
+					   % (timep,), 1, errlevel = 0)
 
 	times_in_secs.sort()
 	inc_pretty_time = "\n".join(map(Time.timetopretty, times_in_secs))
@@ -785,16 +654,17 @@ def rot_check_time(time_string):
 		Log.FatalError("Found %d relevant increments, dated:\n%s"
 			"\nIf you want to delete multiple increments in this way, "
 			"use the --force." % (len(times_in_secs), inc_pretty_time))
+
 	if len(times_in_secs) == 1:
 		Log("Deleting increment at time:\n" + inc_pretty_time, 3)
 	else: Log("Deleting increments at times:\n" + inc_pretty_time, 3)
-	return times_in_secs[-1]+1 # make sure we don't delete current increment
+	manage.delete_earlier_than(Globals.rbdir, time)
 
 def rot_require_rbdir_base(rootrp):
 	"""Make sure pointing to base of rdiff-backup dir"""
 	if restore_index != ():
-		Log.FatalError("Increments for directory %s cannot be removed "
-					   "separately.\nInstead run on entire directory %s." %
+		Log.FatalError("Increments for directory %s cannot be removed separately.\n"
+					   "Instead run on entire directory %s." %
 					   (rootrp.path, restore_root.path))
 
 
